@@ -255,10 +255,10 @@ class _OptFormatter(object):
         lines = self._format_help(help_text)
 
         if getattr(opt.type, 'min', None) is not None:
-            lines.append('# Minimum value: %d\n' % opt.type.min)
+            lines.append('# Minimum value: {}\n'.format(opt.type.min))
 
         if getattr(opt.type, 'max', None) is not None:
-            lines.append('# Maximum value: %d\n' % opt.type.max)
+            lines.append('# Maximum value: {}\n'.format(opt.type.max))
 
         if getattr(opt.type, 'choices', None):
             lines.append('# Possible values:\n')
@@ -321,6 +321,14 @@ class _OptFormatter(object):
                 '# effect on stability and/or performance.\n'
             )
 
+        if opt.sample_default:
+            lines.append(
+                '#\n'
+                '# This option has a sample default set, which means that\n'
+                '# its actual default value may vary from the one documented\n'
+                '# below.\n'
+            )
+
         if hasattr(opt.type, 'format_defaults'):
             defaults = opt.type.format_defaults(opt.default,
                                                 opt.sample_default)
