@@ -25,6 +25,7 @@ import re
 import warnings
 
 import abc
+from debtcollector import removals
 import netaddr
 import rfc3986
 import six
@@ -43,7 +44,10 @@ class ConfigType(object):
 
         """
         if sample_default is not None:
-            default_str = sample_default
+            if isinstance(sample_default, six.string_types):
+                default_str = sample_default
+            else:
+                default_str = self._formatter(sample_default)
         elif default is None:
             default_str = self.NONE_DEFAULT
         else:
@@ -140,7 +144,7 @@ class String(ConfigType):
         value = str(value)
         if self.quotes and value:
             if value[0] in "\"'":
-                if value[-1] != value[0]:
+                if len(value) == 1 or value[-1] != value[0]:
                     raise ValueError('Non-closed quote: %s' % value)
                 value = value[1:-1]
 
@@ -347,7 +351,7 @@ class Number(ConfigType):
         )
 
     def _formatter(self, value):
-        return str(value)
+        return six.text_type(value)
 
 
 class Integer(Number):
@@ -457,7 +461,8 @@ class List(ConfigType):
     comma and next item until validation succeeds or there is no parts left.
     In the later case it will signal validation error.
 
-    :param item_type: type of list items
+    :param item_type: Type of list items. Should be an instance of
+                      ``ConfigType``.
     :param bounds: if True, value should be inside "[" and "]" pair
     :param type_name: Type name to be used in the sample config file.
 
@@ -525,14 +530,16 @@ class List(ConfigType):
         )
 
     def _formatter(self, value):
+        fmtstr = '[{}]' if self.bounds else '{}'
         if isinstance(value, six.string_types):
-            return value
+            return fmtstr.format(value)
         if isinstance(value, list):
             value = [
-                six.text_type(v)
+                self.item_type._formatter(v)
                 for v in value
             ]
-        return ','.join(value)
+            return fmtstr.format(','.join(value))
+        return fmtstr.format(self.item_type._formatter(value))
 
 
 class Range(ConfigType):
@@ -803,7 +810,7 @@ class Hostname(ConfigType):
         return value
 
 
-class HostAddress(object):
+class HostAddress(ConfigType):
     """Host Address type.
 
     Represents both valid IP addresses and valid host domain names
@@ -821,6 +828,7 @@ class HostAddress(object):
 
         """
 
+        super(HostAddress, self).__init__(type_name=type_name)
         self.ip_address = IPAddress(version, type_name)
         self.hostname = Hostname('localhost')
 
@@ -876,31 +884,54 @@ class URI(ConfigType):
         self.schemes = schemes
 
     def __call__(self, value):
-        if not rfc3986.is_valid_uri(value, require_scheme=True,
-                                    require_authority=True):
-            raise ValueError('invalid URI: %r' % value)
+        uri = rfc3986.uri_reference(value)
+        validator = rfc3986.validators.Validator().require_presence_of(
+            'scheme', 'host',
+        ).check_validity_of(
+            'scheme', 'host', 'path',
+        )
+        if self.schemes:
+            validator = validator.allow_schemes(*self.schemes)
+        try:
+            validator.validate(uri)
+        except rfc3986.exceptions.RFC3986Exception as exc:
+            raise ValueError(exc)
 
         if self.max_length is not None and len(value) > self.max_length:
             raise ValueError("Value '%s' exceeds maximum length %d" %
                              (value, self.max_length))
 
-        if self.schemes:
-            scheme = rfc3986.uri_reference(value).scheme
-            if scheme not in self.schemes:
-                raise ValueError("URI scheme '%s' not in %s" %
-                                 (scheme, self.schemes))
-
-        self.value = value
+        # NOTE(dhellmann): self.value is deprecated, and we don't want
+        # to trigger a deprecation warning ourselves so we modify
+        # self._value directly.
+        self._value = value
         return value
 
+    @removals.removed_property
+    def value(self):
+        return self._value
+
+    @value.setter
+    def value(self, newval):
+        self._value = newval
+
+    @value.deleter
+    def value(self):
+        del self._value
+
     def __repr__(self):
         return 'URI'
 
     def __eq__(self, other):
-        return (
-            (self.__class__ == other.__class__) and
-            (self.value == other.value)
+        to_compare = ['__class__', 'max_length', 'schemes']
+        unset = object()
+        my_values = tuple(
+            getattr(self, name, unset) for name in to_compare
         )
+        other_values = tuple(
+            getattr(other, name, unset) for name in to_compare
+        )
+        return my_values == other_values
 
     def _formatter(self, value):
         return value
